import "package:device_info_plus/device_info_plus.dart";
import "package:feedback_repository_interface/feedback_repository_interface.dart";
import "package:flutter/material.dart";
import "package:flutter_feedback/src/config/feedback_translations.dart";
import "package:flutter_feedback/src/widgets/star_rating.dart";
import "package:package_info_plus/package_info_plus.dart";

///
Future<void> showFeedbackBottomSheet({
  required BuildContext context,
  required Future<void> Function(Report report) onFeedback,
  VoidCallback? onGoodRating,
}) async {
  var rating = 0;
  var currentStep = ValueNotifier(1);
  await showModalBottomSheet(
    isScrollControlled: true,
    backgroundColor: Colors.transparent,
    context: context,
    builder: (BuildContext context) {
      return Padding(
        padding: EdgeInsets.only(
          bottom: MediaQuery.of(context).viewInsets.bottom,
        ),
        child: ValueListenableBuilder<int>(
          valueListenable: currentStep,
          builder: (context, value, child) {
            switch (value) {
              case 1:
                return RateApp(
                  onRatingChanged: (stars) async {
                    rating = stars;
                    if (stars >= 4 && onGoodRating != null) {
                      onGoodRating();
                      Navigator.of(context).pop();
                      return;
                    }
                    await Future.delayed(const Duration(milliseconds: 500));
                    currentStep.value = 2;
                  },
                );
              case 2:
                return GiveFeedback(
                  onFeedbackGiven: (value) async {
                    var packageInfo = await PackageInfo.fromPlatform();
                    var deviceInfo = await DeviceInfoPlugin().deviceInfo;
                    String model;
                    String osVersion;
                    if (deviceInfo is AndroidDeviceInfo) {
                      model = deviceInfo.data["model"];
                      osVersion =
                          // ignore: avoid_dynamic_calls
                          deviceInfo.data["version"]["sdkInt"].toString();
                    } else {
                      model = deviceInfo.data["name"];
                      osVersion = deviceInfo.data["systemVersion"].toString();
                    }
                    var report = Report(
                      appVersion: packageInfo.version,
                      deviceModel: model,
                      osVersion: osVersion,
                      rating: rating,
                      feedback: value,
                    );
                    await onFeedback(report);

                    currentStep.value = 3;
                  },
                );
              case 3:
                return const FeedbackThankYou();
            }
            return const SizedBox();
          },
        ),
      );
    },
  );
}

///
class RateApp extends StatefulWidget {
  ///
  const RateApp({
    required this.onRatingChanged,
    this.translations = const FeedbackTranslations.empty(),
    super.key,
  });

  ///
  final FeedbackTranslations translations;

  ///
  final void Function(int) onRatingChanged;

  @override
  State<RateApp> createState() => _RateAppState();
}

class _RateAppState extends State<RateApp> {
  int rating = 0;
  @override
  Widget build(BuildContext context) {
    var theme = Theme.of(context);
    return Container(
      decoration: const BoxDecoration(
        color: Colors.white,
        borderRadius: BorderRadius.only(
          topLeft: Radius.circular(16),
          topRight: Radius.circular(16),
        ),
      ),
      padding: const EdgeInsets.all(24),
      child: Column(
        mainAxisSize: MainAxisSize.min,
        crossAxisAlignment: CrossAxisAlignment.center,
        children: <Widget>[
          Text(
            widget.translations.ratingQuestion,
            style: theme.textTheme.titleMedium,
            textAlign: TextAlign.center,
          ),
          const SizedBox(height: 20),
          StarRating(
            onRatingChanged: (stars) {
              widget.onRatingChanged(stars);
              setState(() {
                rating = stars;
              });
            },
          ),
          TextButton(
            onPressed: () {
              Navigator.of(context).pop();
            },
            child: Text(
              widget.translations.maybeLater,
              style: theme.textTheme.displaySmall
                  ?.copyWith(decoration: TextDecoration.underline),
            ),
          ),
        ],
      ),
    );
  }
}

///
class GiveFeedback extends StatefulWidget {
  ///
  const GiveFeedback({
    required this.onFeedbackGiven,
    this.translations = const FeedbackTranslations.empty(),
    super.key,
  });

  ///
  final FeedbackTranslations translations;

  ///
  final void Function(String? feedback) onFeedbackGiven;

  @override
  State<GiveFeedback> createState() => _GiveFeedbackState();
}

class _GiveFeedbackState extends State<GiveFeedback> {
  String? feedback;
  GlobalKey<FormState> formKey =
      GlobalKey<FormState>(debugLabel: "feedback_form");

  @override
  Widget build(BuildContext context) {
    var theme = Theme.of(context);
    return Container(
      decoration: const BoxDecoration(
        color: Colors.white,
        borderRadius: BorderRadius.only(
          topLeft: Radius.circular(16),
          topRight: Radius.circular(16),
        ),
      ),
      padding: const EdgeInsets.all(24),
      child: Form(
        key: formKey,
        child: Column(
          mainAxisSize: MainAxisSize.min,
          crossAxisAlignment: CrossAxisAlignment.center,
          children: <Widget>[
            Text(
              widget.translations.helpUsImprove,
              style: theme.textTheme.titleMedium,
              textAlign: TextAlign.center,
            ),
            const SizedBox(height: 12),
            Flexible(
              child: TextFormField(
                textInputAction: TextInputAction.done,
                key: const Key("feedback_text_field"),
                expands: false,
                maxLines: null,
                minLines: null,
                maxLength: 200,
                decoration: InputDecoration(
                  contentPadding: const EdgeInsets.all(16),
                  hintStyle: theme.textTheme.bodySmall?.copyWith(
                    color: theme.textTheme.bodySmall?.color?.withOpacity(0.5),
                  ),
                  hintText: widget.translations.feedbackHintText,
                  border: const OutlineInputBorder(
                    borderRadius: BorderRadius.all(Radius.circular(16)),
                  ),
                ),
                validator: (value) {
                  if (value == null || value.isEmpty) {
                    return widget.translations.feedbackValidationMessage;
                  }
                  return null;
                },
                onChanged: (value) {
                  setState(() {
                    feedback = value;
                  });
                },
              ),
            ),
            const SizedBox(height: 32),
            Row(
              children: [
                Expanded(
                  child: Padding(
                    padding: const EdgeInsets.symmetric(horizontal: 56),
                    child: FilledButton(
                      onPressed: () {
                        if (!formKey.currentState!.validate()) {
                          return;
                        }
                        widget.onFeedbackGiven(feedback);
                      },
                      child: Padding(
                        padding: const EdgeInsets.all(8),
                        child: Text(
                          widget.translations.feedbackSubmit,
                          style: theme.textTheme.displayLarge,
                        ),
                      ),
                    ),
                  ),
                ),
              ],
            ),
            TextButton(
              onPressed: () {
                widget.onFeedbackGiven(null);
              },
              child: Text(
                widget.translations.skip,
                style: theme.textTheme.displaySmall
                    ?.copyWith(decoration: TextDecoration.underline),
              ),
            ),
          ],
        ),
      ),
    );
  }
}

///
class FeedbackThankYou extends StatelessWidget {
  ///
  const FeedbackThankYou({
    this.translations = const FeedbackTranslations.empty(),
    super.key,
  });

  ///
  final FeedbackTranslations translations;
  @override
  Widget build(BuildContext context) {
    var theme = Theme.of(context);

    return Container(
      decoration: const BoxDecoration(
        color: Colors.white,
        borderRadius: BorderRadius.only(
          topLeft: Radius.circular(16),
          topRight: Radius.circular(16),
        ),
      ),
      padding: const EdgeInsets.all(24),
      child: Column(
        mainAxisSize: MainAxisSize.min,
        crossAxisAlignment: CrossAxisAlignment.center,
        children: <Widget>[
          Text(
            translations.feedbackThankYoutitle,
            style: theme.textTheme.titleMedium,
            textAlign: TextAlign.center,
          ),
          const SizedBox(height: 20),
          Row(
            children: [
              Expanded(
                child: Padding(
                  padding: const EdgeInsets.symmetric(horizontal: 56),
                  child: FilledButton(
                    onPressed: () {
                      Navigator.of(context).pop();
                    },
                    child: Padding(
                      padding: const EdgeInsets.all(8),
                      child: Text(
                        translations.feedbackClose,
                        style: theme.textTheme.displayLarge,
                      ),
                    ),
                  ),
                ),
              ),
            ],
          ),
          const SizedBox(height: 20),
        ],
      ),
    );
  }
}
